// Fifo Handling
#ifndef MJ_FIFOSIZE
#define MJ_FIFOSIZE 64
#endif

struct mj_keybuffer {
  unsigned char c;  // the Character
  unsigned char d;  // the Delay in Bits 0-3, Bit 7 = "Special Char" which will not be auto-released
};

mj_keybuffer mj_fifostore[MJ_FIFOSIZE];
int mj_fifoindex_nextout = 0;
int mj_fifoindex_nextin = 0;


// Get one fifo entry
mj_keybuffer mj_fifo_getelement() {
  int index;
  if (! (mj_fifoindex_nextout == mj_fifoindex_nextin)) {
    index = mj_fifoindex_nextout;
    if (mj_fifoindex_nextout == (MJ_FIFOSIZE - 1)) {
      mj_fifoindex_nextout = 0;
    } else {
      mj_fifoindex_nextout++;
    }
    return mj_fifostore[index];;
  } else {
    return ((mj_keybuffer) {
      0, 0
    });
  }
}

// tell how many entries are left in fifo
int mj_fifo_stored() {
  if (mj_fifoindex_nextin >= mj_fifoindex_nextout) {
    return (mj_fifoindex_nextin - mj_fifoindex_nextout);
  } else {
    return ((MJ_FIFOSIZE - mj_fifoindex_nextout) + mj_fifoindex_nextin);
  }
}

// put an element into fifo
bool mj_fifo_putelement(mj_keybuffer b) {
  int index;
  index = mj_fifoindex_nextin;
  if (index == (MJ_FIFOSIZE - 1)) {
    index = 0;
  } else {
    index++;
  }
  if (index == mj_fifoindex_nextout) {
    return (false);
  } else {
    mj_fifostore[mj_fifoindex_nextin] = b;
    mj_fifoindex_nextin = index;
  }
  return (true);
}

// Now more MJ Specific Code - above is mostly generic.

const unsigned char MJ_KEYPAD[] = { 234, 225, 226, 227, 228, 229, 230, 231, 232, 233 };
const unsigned char MJ_TRANSLATE[] = {'!', '"', '#', '$', '%', '&', 39, '(', ')', '*',
                                      '+', ',', '-', '.', '/', ':', ';', '<', '=', '>',
                                      '?', '@', 'A', 'Q', 'Y', 'Z', '[', ']', '\\', '^',
                                      '_', '`', 'a', 'q', 'y', 'z', '{', '|', '}', '~'
                                     };

void putfifo(unsigned char c, unsigned char d) {
  mj_fifo_putelement((mj_keybuffer) {
    c, (d & 0x0f)
  });
}

void putfifo_special(unsigned char c, unsigned char d) {
  mj_fifo_putelement((mj_keybuffer) {
    c, (d | 0x80)
  });
}

// Adds Character to HID Fifo and translate "problematic" Chars to their Alt-<ascii> ones which should work with nearly any key-layout on windows or DOS
void putfifo_translate(unsigned char c, int d) {
  unsigned int i;
  int found = 0;
  for (i = 0; i < sizeof(MJ_TRANSLATE); i++ ) {
    if (c == MJ_TRANSLATE[i]) {
      found = 1;
      break;
    }
  }
  if (found == 1) {
    putfifo_special(KEY_LEFT_ALT, 1);
    if (c > 100) {
      putfifo(MJ_KEYPAD[c / 100], 1);
    }
    putfifo(MJ_KEYPAD[(c / 10) % 10], 1);
    putfifo(MJ_KEYPAD[c % 10], 1);
    putfifo_special(KEY_RELEASE, d);
  } else {
    putfifo(c, d);
  }
}


// Put a string into fifo with interpreting it
// type = 1 for string in c and 0 for eeprom from offset MJ_EEPROM_CONTENT
void putfifo_string(char *c, int type) {
  unsigned int len, i = 0;
  int d;
  unsigned char ch;
  d = mj_keydelay_default;
  if (type == 0) {
    len = mj_usedbytes;
  } else {
    len = strlen((const char*) c);
  }
  if (len >= MJ_FIFOSIZE) {
    len = MJ_FIFOSIZE - 1;
  }
  for (i = 0; i < len; i++) {
    if ( mj_getvalue(i, type, c) == '\\' && (i + 1) < len ) {
      switch (mj_getvalue(i + 1, type, c)) {
        case 'e': // ESC
          putfifo(KEY_ESC, d);
          break;
        case 'n': // RETURN
        case 'r':
          putfifo(KEY_RETURN, d);
          break;
        case 't': // TAB
          putfifo(KEY_TAB, d);
          break;
        case '\\': // '\'
          putfifo_translate('\\', d);
          break;
        case 0x27: // '
          putfifo_translate(0x27, d);
          break;
        case '"':
          putfifo_translate('"', d);
          break;
        case '?':
          putfifo_translate('?', d);
          break;
        case '0':
          putfifo_translate(0, d);
          break;
        case 'd': // Set new delay for following keys
          if ((i + 2) < len) {
            ch = mj_getvalue(i + 2, type, c);
            if ( ch >= '0' && ch <= '9' ) {
              d = (int) ch - '0';
            }
          }
          i++;
          break;
        case 'x': // Hex Code - invalid Codes will be silently skipped (Not 0-9/a-f/A-F)
          if ((i + 3) < len) {
            putfifo_translate(mj_convert_hex_char( mj_getvalue(i + 2, type, c) , mj_getvalue(i + 3, type, c)), d);
            i += 2;
          }
          break;
        case 'X': // Hex Code - like above but with no translation
          if ((i + 3) < len) {
            putfifo(mj_convert_hex_char( mj_getvalue(i + 2, type, c) , mj_getvalue(i + 3, type, c)), d);
            i += 2;
          }
          break;
        case 's': // Hex Code - like above but for "Special Keys" (Only Press, no Auto-Release - \sff = Release all Keys
          if ((i + 3) < len) {
            putfifo_special(mj_convert_hex_char( mj_getvalue(i + 2, type, c) , mj_getvalue(i + 3, type, c)), d);
            i += 2;
          }
          break;
        default:
          putfifo_translate('\\', d);
          putfifo_translate(mj_getvalue(i + 1, type, c), d);
          break;
      }
      i++;
    } else {
      putfifo_translate(mj_getvalue(i, type, c), d);
    }
  }
}
