// LED Options Parser
// p = Pin Assignment. Example atlp113 sets LED1 Pin to 13. Pin number may not be higher than available pins
//           New Pin will be checked for collision with other LED Pin and Button Pin.
//     atlps Shows Pin Assingment
//     atlpS Saves Pin Assignment to EEPROM
//     atlpr resets Pins to default if no Collision with key pins
// t = LED Timing-Modifier. atlt110000 sets the Modifier 1 to 10000 (default), atlts Shows Timings, atltS Saves Timings to EEPROM, atltr restores Default Timings
// m = LED Mode. atlm<decimal value> sets the Mode, atlms shows mode, atlmS Saves Mode to EEPROM, atlmr restores Default LED Mode
// o = LED Option. atlo<4 hex digits> sets the options. atlor restores Default Options
void parseled(char *c, int num) {
  unsigned int value;
  int n;
  bool x;
  switch (c[0]) {
    case 'p':
      if (num >= 3) {
        switch (c[1]) {
          case '1':
          case '2':
            value = (unsigned int) mj_convert_dec_long(c + 2);
            Serial.print(value);
            Serial.print(F("\r\n"));
            // Check collision with other Pins already used (Key Inputs)
            x = true;
            for (n = 0; n < 4; n++) {  // Avoid collision with our 4 Key-Pins
              if (value == mj_keypin[n]) {
                x = false;
              }
            }
#ifndef LED_BUILTIN_RX   // For example Arduino Uno - if we'd use the RX/TX Pins for LED here we kill the serial communication. Leonardo works with 0/1
            if (value == 0 || value == 1) {
              x = false;
            }
#endif
            if (c[1] == '1') {
              if (value == mj_ledpin2) {
                x = false;
              }
            } else {
              if (value == mj_ledpin1) {
                x = false;
              }
            }
            if (x == true && value < NUM_DIGITAL_PINS ) {
              pinMode(value, OUTPUT);
              if (c[1] == '1') {
                digitalWrite(mj_ledpin1, 0);
                mj_ledpin1 = value;
              } else {
                digitalWrite(mj_ledpin2, 0);
                mj_ledpin2 = value;
              }
            }
            break;
        }
      } else if (num == 2) {
        switch (c[1]) {
          case 'r': // Restore LED Default Pins
            if ((MJ_DEFAULT_LEDPIN_1 != mj_keypin[0]) && (MJ_DEFAULT_LEDPIN_1 != mj_keypin[1]) && (MJ_DEFAULT_LEDPIN_1 != mj_keypin[2]) && (MJ_DEFAULT_LEDPIN_1 != mj_keypin[3])) {
              mj_ledpin1 = MJ_DEFAULT_LEDPIN_1;
            }
            if ((MJ_DEFAULT_LEDPIN_2 != mj_keypin[0]) && (MJ_DEFAULT_LEDPIN_2 != mj_keypin[1]) && (MJ_DEFAULT_LEDPIN_2 != mj_keypin[2]) && (MJ_DEFAULT_LEDPIN_2 != mj_keypin[3])) {
              mj_ledpin2 = MJ_DEFAULT_LEDPIN_2;
            }
            pinMode(mj_ledpin1, OUTPUT);
            pinMode(mj_ledpin2, OUTPUT);
            [[fallthrough]];
          case 's': // Show serial Pin Assignment
            Serial.print(F("LEDPIN1: "));
            Serial.print(mj_ledpin1);
            Serial.print(F(" LEDPIN2: "));
            Serial.print(mj_ledpin2);
            Serial.print(F("\r\n"));
            break;
          case 'S': // Save Serial Pin Assignment to EEPROM
            mj_eeprom_write_int(MJ_EEPROM_LEDPIN_1, mj_ledpin1); // Save Pin 1
            mj_eeprom_update_value(MJ_EEPROM_LEDPIN_2, 2, mj_ledpin2); // Save Pin 2 plus recalculate + Update CRC
            break;
          default:
            break;
        }
      }
      break;
    case 't': // LED Timing Modifiers
      if (num >= 2) {
        switch (c[1]) {
          case '1':
          case '2':
          case '3':
          case '4':
            if (num > 2) {
              value = (unsigned int) mj_convert_dec_long(c + 2);
              n = c[1] - '1';
              mj_ledtiming[n] = value;
              mj_ledtiming_changed = true; // Force Recalculation of LED Timings
            }
            break;
          case 'r': // Restore Default Timings
            mj_ledtiming[0] = MJ_DEFAULT_LEDTIMING_1;
            mj_ledtiming[1] = MJ_DEFAULT_LEDTIMING_2;
            mj_ledtiming[2] = MJ_DEFAULT_LEDTIMING_3;
            mj_ledtiming[3] = MJ_DEFAULT_LEDTIMING_4;
            mj_ledtiming_changed = true;
            [[fallthrough]];
          case 's':
            Serial.print(F("LED Timing"));
            for (n = 0; n < 4; n++) {
              Serial.print(n + 1);
              Serial.print(F(":"));
              Serial.print(mj_ledtiming[n]);
              Serial.write(' ');
            }
            Serial.print(F("\r\n"));
            break;
          case 'S': // Save LED Timings to EEPROM
            mj_eeprom_write_int(MJ_EEPROM_LEDTIMING_1, mj_ledtiming[0]);
            mj_eeprom_write_int(MJ_EEPROM_LEDTIMING_2, mj_ledtiming[1]);
            mj_eeprom_write_int(MJ_EEPROM_LEDTIMING_3, mj_ledtiming[2]);
            mj_eeprom_update_value(MJ_EEPROM_LEDTIMING_4, 2, mj_ledtiming[3]);
            break;
          default:
            break;
        }
      }
      break;
    case 'm': // LED Mode
      if (num >= 2) {
        switch (c[1]) {
          case '0':
          case '1':
          case '2':
          case '3':
          case '4':
          case '5':
          case '6':
          case '7':
          case '8':
          case '9':
            value = (unsigned int) mj_convert_dec_long(c + 1);
            mj_ledmode = value;
            mj_ledtiming_changed = true; // If LED Mode changes then recalculate Timings in LED Process
            break;
          case 'r':
            mj_ledmode = MJ_DEFAULT_LEDMODE;
            mj_ledtiming_changed = true; // If LED Mode changes then recalculate Timings in LED Process
            [[fallthrough]];
          case 's':
            Serial.print(F("LED Mode: "));
            Serial.print(mj_ledmode);
            Serial.print(F("\r\n"));
            break;
          case 'S':
            mj_eeprom_update_value(MJ_EEPROM_LEDMODE, 2, mj_ledmode);
            break;
          default:
            break;
        }
      }
      break;
    case 'o': // LED Options
      if (num >= 2) {
        switch (c[1]) {
          case '0':
          case '1':
          case '2':
          case '3':
          case '4':
          case '5':
          case '6':
          case '7':
          case '8':
          case '9':
          case 'a':
          case 'b':
          case 'c':
          case 'd':
          case 'e':
          case 'f':
          case 'A':
          case 'B':
          case 'C':
          case 'D':
          case 'E':
          case 'F':
            value = (unsigned int) mj_convert_hex_long(c + 1);
            mj_ledoption = value;
            mj_ledtiming_changed = true; // If LED Option changes then recalculate Timings in LED Process
            break;
          case 'r':
            mj_ledoption = MJ_DEFAULT_LEDOPTION;
            mj_ledtiming_changed = true; // If LED Option changes then recalculate Timings in LED Process
            [[fallthrough]];
          case 's':
            Serial.print(F("LED Option: 0x"));
            Serial.print(mj_ledoption, HEX);
            Serial.print(F("\r\n"));
            break;
          case 'S':
            mj_eeprom_update_value(MJ_EEPROM_LEDOPTION, 2, mj_ledoption);
            break;
          default:
            break;
        }
      }
      break;
  }
}

// Simply prints the info message to serial console
void parseinfo() {
  Serial.print(F(INFOSTRING));
}

// Commands ate...
// d = dump eeprom contents
// c = Calc CRC and show calculated and stored CRC
// C = Calc CRC and update stored CDC
// i,I,z = initialize EEPROM Contents - need to be given 3 times - i just rewrites known content, I wipes all, z only zeroes whole eeprom (0xff) but does not load defaults
// w,W = Write value b/c = byte, i = int, l = long to given address (3 hex digits) with value (1-8 digits). byte = 2, int = 4, long = 1-8 digits W with update CRC, w without update CRC
// r,R = Read value from address (same as w/W but without value) and show

void parseeeprom(char *c, int num) {
  unsigned long crc_calc, crc_stored;
  static int i = 2;
  unsigned long v;
  int t = 0, a = 0;
  switch (c[0]) {
    case 'i':
    case 'I':
    case 'z':
      ;
      break;
    default:
      i = 2;
      break;
  }
  switch (c[0]) {
    case 'd':
      mj_eeprom_dump_serial();
      break;
    case 'c':
      crc_calc = mj_eeprom_calc_crc(MJ_EEPROM_CRCAREA_START, MJ_EEPROM_CRCAREA_END);
      crc_stored = mj_eeprom_read_long(MJ_EEPROM_CRC);
      Serial.print(F("EEPROM CRC "));
      Serial.write((crc_calc == crc_stored) ? "valid" : "invalid");
      Serial.print(F(" stored: "));
      Serial.print(crc_stored);
      Serial.print(F(" calculated: "));
      Serial.print(crc_calc);
      Serial.print(F("\r\n"));
      break;
    case 'C':
      crc_calc = mj_eeprom_calc_crc(MJ_EEPROM_CRCAREA_START, MJ_EEPROM_CRCAREA_END);
      mj_eeprom_write_long(MJ_EEPROM_CRC, crc_calc);
      break;
    case 'i':
    case 'I':
    case 'z':
      if (i > 0) {
        Serial.print(F("Please repeat command "));
        Serial.print((i == 2) ? "twice" : "once");
        Serial.print(F(" to wipe EEPROM\r\n"));
        i--;
      } else {
        i = 2;
        if (c[0] == 'I' || c[0] == 'z') {
          Serial.print(F("Wiping\r\n"));
          mj_eeprom_wipe();
        }
        if (c[0] == 'I' || c[0] == 'i') {
          Serial.print(F("Initializing defaults\r\n"));
          mj_eeprom_init_defaults();
          mj_eeprom_get_config();
        }
      }
      break;
    case 'w':
    case 'W':  // Write wants to have 5,7 or 11 Hex digits depending on modifier: atwb 5, atwi 7, atwl 11. first 3 digits address, remaining value.
      if (num >= 6) {
        a = mj_convert_hex_int('0', c[2], c[3], c[4]);
      }
      switch (c[1]) {
        case 'b':
        case 'c': // One char
          if (num == 7) {
            v = (unsigned long) mj_convert_hex_char(c[5], c[6]);
            t = 1;
          }
          break;
        case 'i': // int
          if (num == 9) {
            v = (unsigned long) mj_convert_hex_int(c[5], c[6], c[7], c[8]);
            t = 2;
          }
          break;
        case 'l': // long
          if (num >= 6) {
            v = mj_convert_hex_long(c + 5);
            t = 4;
          }
          break;
      }
      if (t > 0) {
        switch (t) {
          case 1:
            mj_eeprom_write_byte(a, (byte) v);
            break;
          case 2:
            mj_eeprom_write_int(a, (unsigned int) v);
            break;
          case 4:
            mj_eeprom_write_long(a, v);
            break;
          default:
            break;
        }
        if (c[0] == 'W') {
          crc_calc = mj_eeprom_calc_crc(MJ_EEPROM_CRCAREA_START, MJ_EEPROM_CRCAREA_END);
          mj_eeprom_write_long(MJ_EEPROM_CRC, crc_calc);
        }
      }
      break;
    case 'r':
    case 'R':  // Read wants to have 3 hex digits address.
      if (num == 5) {
        a = mj_convert_hex_int('0', c[2], c[3], c[4]);
      }
      switch (c[1]) {
        case 'b':
        case 'c': // One char
          v = (unsigned long) mj_eeprom_read_byte(a);
          t = 1;
          break;
        case 'i': // int
          v = (unsigned long) mj_eeprom_read_int(a);
          t = 2;
          break;
        case 'l': // long
          v = mj_eeprom_read_long(a);
          t = 4;
          break;
        default:
          break;
      }
      if (t > 0) {
        Serial.print(F("Addr: "));
        Serial.print(a, HEX);
        Serial.print(F(" value: "));
        Serial.print(v, HEX);
        Serial.print(F("\r\n"));
      }
      break;
  }
}

// Command att...
// atts - print eeprom content to serial
// attk - write string after k to keyboard
// attw - overwrite string in eeprom - if no string is given this will mark the string deleted
// attt - type string in eeprom
// attd<0-9><decimal> - configure typedelay <0-9> to <decimal> ms
// attds - show active typedelays
// attdr - reset typedelays to default
// attdS - save active typedelays to EEPROM
// attar - reset autotype delay to default
// attas - show autotype delay
// attat<decimal> - set autotype delay to <decimal> in ms
// attad - disable autotyping
// attae - enable autotyping
// attaS - Save autotype delay to EEPROM

void parsekeyboard(char *c, int num) {
  int i = 0;
  unsigned long value = 0;
  switch (c[0]) {
    case 'a':
      if (num > 2 && c[1] == 't') {
        mj_autotype_delay = mj_convert_dec_long(c + 2);
      } else if (num == 2) {
        switch (c[1]) {
          case 'r':
            mj_autotype_delay = MJ_DEFAULT_AUTOTYPE_DELAY;
            mj_autotype = MJ_DEFAULT_AUTOTYPE;
            [[fallthrough]];
          case 's':
            Serial.print(F("AutoType is "));
            Serial.print(mj_autotype > 0 ? "en" : "dis");
            Serial.print(F("abled with delay "));
            Serial.print(mj_autotype_delay);
            Serial.print(F("\r\n"));
            break;
          case 'S':
            mj_eeprom_write_byte(MJ_EEPROM_AUTOTYPE, mj_autotype);
            mj_eeprom_update_value(MJ_EEPROM_AUTOTYPE_DELAY, 4, mj_autotype_delay);
            break;
          case 'e':
            mj_autotype = true;
            break;
          case 'd':
            mj_autotype = false;
            break;
          default:
            break;
        }
      }
      break;
    case 's':
      for (i = 0; i < mj_usedbytes && i < MJ_EEPROM_CONTENT_SIZE; i++) {
        Serial.write((char) mj_eeprom_read_byte(i + MJ_EEPROM_CONTENT));
      }
      Serial.print(F("\r\n"));
      break;
    case 'k':
      putfifo_string(c + 1, 1);
      break;
    case 'w':
      for (i = 0; (c[i + 1] > 0) && ( i < (num - 1) ) && (i < MJ_EEPROM_CONTENT_SIZE); i++) {
        mj_eeprom_write_byte(MJ_EEPROM_CONTENT + i, c[i + 1]);
      }
      mj_eeprom_write_byte(MJ_EEPROM_CONTENT + i, '\0');
      mj_usedbytes = i;
      mj_eeprom_update_value(MJ_EEPROM_USEDBYTES, 2, mj_usedbytes);
      break;
    case 't':
      putfifo_string(c, 0);
      break;
    case 'd':
      if (num == 2) {
        switch (c[1]) {
          case 'r': // reset eeprom defaults;
            mj_keydelay[0] = MJ_DEFAULT_KEYDELAY_0;
            mj_keydelay[1] = MJ_DEFAULT_KEYDELAY_1;
            mj_keydelay[2] = MJ_DEFAULT_KEYDELAY_2;
            mj_keydelay[3] = MJ_DEFAULT_KEYDELAY_3;
            mj_keydelay[4] = MJ_DEFAULT_KEYDELAY_4;
            mj_keydelay[5] = MJ_DEFAULT_KEYDELAY_5;
            mj_keydelay[6] = MJ_DEFAULT_KEYDELAY_6;
            mj_keydelay[7] = MJ_DEFAULT_KEYDELAY_7;
            mj_keydelay[8] = MJ_DEFAULT_KEYDELAY_8;
            mj_keydelay[9] = MJ_DEFAULT_KEYDELAY_9;
            [[fallthrough]];
          case 's': // show delays
            Serial.print(F("KeyType Delay List"));
            for (i = 0; i < 10; i++) {
              Serial.write(' ');
              Serial.print(i);
              Serial.print(F(":"));
              Serial.print(mj_keydelay[i]);
            }
            Serial.print(F("\r\n"));
            break;
          case 'S':
            mj_eeprom_write_long(MJ_EEPROM_KEYDELAY_0, mj_keydelay[0]);
            mj_eeprom_write_long(MJ_EEPROM_KEYDELAY_1, mj_keydelay[1]);
            mj_eeprom_write_long(MJ_EEPROM_KEYDELAY_2, mj_keydelay[2]);
            mj_eeprom_write_long(MJ_EEPROM_KEYDELAY_3, mj_keydelay[3]);
            mj_eeprom_write_long(MJ_EEPROM_KEYDELAY_4, mj_keydelay[4]);
            mj_eeprom_write_long(MJ_EEPROM_KEYDELAY_5, mj_keydelay[5]);
            mj_eeprom_write_long(MJ_EEPROM_KEYDELAY_6, mj_keydelay[6]);
            mj_eeprom_write_long(MJ_EEPROM_KEYDELAY_7, mj_keydelay[7]);
            mj_eeprom_write_long(MJ_EEPROM_KEYDELAY_8, mj_keydelay[8]);
            mj_eeprom_update_value(MJ_EEPROM_KEYDELAY_9, 4, mj_keydelay[9]);
            break;
        }
      } else if (num >= 3) {
        if (c[1] >= '0' && c[1] <= '9') {
          i = c[1] - '0';
          value = mj_convert_dec_long(c + 2);
          mj_keydelay[i] = value;
        }
      }
    default:
      break;
  }
}

// Command atb
// atb<1-4><decimal> set button 4 to pin <decimal> - may not be higher than available pins
// atbr - reset button config to default
// atbs - show active button config
// atbS - save active button config to EEPROM
void parsebuttons(char *c, int num) {
  unsigned int value;
  int n = 0, i = 0;
  bool x;
  switch (c[0]) {
    case '1':
    case '2':
    case '3':
    case '4':
      value = (unsigned int) mj_convert_dec_long(c + 1);
      i = c[0] - '1';
      // Check collision with other Pins already used (Key Inputs)
      x = true;
      for (n = 0; n < 4; n++) {
        if ( n != i ) {
          if (value == mj_keypin[n]) {
            x = false;
          }
        }
      }
      if (value == mj_ledpin1) {
        x = false;
      }
      if (value == mj_ledpin2) {
        x = false;
      }
#ifndef LED_BUILTIN_RX   // For example Arduino Uno - if we'd use the RX/TX Pins for Keys here we kill the serial communication. Leonardo works with 0/1
      if (value == 0 || value == 1) {
        x = false;
      }
#endif
      if (x == true && value < NUM_DIGITAL_PINS ) {
        pinMode(value, INPUT_PULLUP);
        mj_keypin[i] = value;
      }
      break;
    case 'r': // Restore Key Default
      if ( (MJ_DEFAULT_KEYPIN_1 != mj_ledpin1) && (MJ_DEFAULT_KEYPIN_1 != mj_ledpin2 ) &&
           (MJ_DEFAULT_KEYPIN_2 != mj_ledpin1) && (MJ_DEFAULT_KEYPIN_2 != mj_ledpin2 ) &&
           (MJ_DEFAULT_KEYPIN_3 != mj_ledpin1) && (MJ_DEFAULT_KEYPIN_3 != mj_ledpin2 ) &&
           (MJ_DEFAULT_KEYPIN_4 != mj_ledpin1) && (MJ_DEFAULT_KEYPIN_4 != mj_ledpin2 ) ) {
        mj_keypin[0] = MJ_DEFAULT_KEYPIN_1;
        mj_keypin[1] = MJ_DEFAULT_KEYPIN_2;
        mj_keypin[2] = MJ_DEFAULT_KEYPIN_3;
        mj_keypin[3] = MJ_DEFAULT_KEYPIN_4;
      }
      pinMode(mj_keypin[0], INPUT_PULLUP);
      pinMode(mj_keypin[1], INPUT_PULLUP);
      pinMode(mj_keypin[2], INPUT_PULLUP);
      pinMode(mj_keypin[3], INPUT_PULLUP);
      [[fallthrough]];
    case 's': // Show serial Pin Assignment
      Serial.print(F("KEYPIN "));
      for (n = 0; n < 4; n++) {
        Serial.print(n + 1);
        Serial.write(':');
        Serial.print(mj_keypin[n]);
        Serial.write(' ');
      }
      Serial.print(F("\r\n"));
      break;
    case 'S': // Save Serial Pin Assignment to EEPROM
      mj_eeprom_write_int(MJ_EEPROM_KEYPIN_1, mj_keypin[0]); // Save Pin 1
      mj_eeprom_write_int(MJ_EEPROM_KEYPIN_2, mj_keypin[1]); // Save Pin 2
      mj_eeprom_write_int(MJ_EEPROM_KEYPIN_3, mj_keypin[2]); // Save Pin 3
      mj_eeprom_update_value(MJ_EEPROM_KEYPIN_4, 2, mj_keypin[3]); // Save Pin 4 plus recalculate + Update CRC
      break;
    default:
      break;
  }
}

// Jiggler options Parser
// atje enable Jiggler
// atjd disable Jiggler
// atjs show Jiggler state
// atjS Save Jiggler state
// atjr recover default Jiggler State
// atjt1<dezimal> Set Minimum Wait-Time between 2 Jiggles to number (0-65535) in ms
// atjt2<dezimal> Set Maximum Wait-Time between 2 Jiggles to number (0-65535) in ms
// atjts Show Jiggler timings
// atjtS Save Jiggler timings to EEPROM
// atjtr recover Jiggler Default State
// atjp<decimal> Set Pixel Max Pixel Movement to number (0-255)
// atjps Show Max Pixel Movement
// atjpS Save Max Pixel Movement to EEPROM
// atjpr recover Max Pixel Movement Default
void parsejiggler(char *c, int num) {
  unsigned int value;
  switch (c[0]) {
    case 'e':
      mj_jiggler_enable = 1;
      mj_ledtiming_changed = true;
      break;
    case 'd':
      mj_jiggler_enable = 0;
      mj_ledtiming_changed = true;
      break;
    case 'r':
      mj_jiggler_enable = MJ_DEFAULT_JIGGLER_ENABLE;
      mj_ledtiming_changed = true;
      [[fallthrough]];
    case 's':
      Serial.print(F("Jiggler "));
      Serial.print(mj_jiggler_enable ? "En" : "Dis");
      Serial.print(F("abled\r\n"));
      break;
    case 'S':
      mj_eeprom_update_value(MJ_EEPROM_JIGGLER_ENABLE, 1, mj_jiggler_enable & 0x01);
      break;
    case 't':
      if (num >= 2) {
        switch (c[1]) {
          case '1':
          case '2':
            if (num >= 3) {
              value = (unsigned int) mj_convert_dec_long(c + 2);
              if (c[1] == '1') {
                if (value <= mj_jiggler_max) {
                  mj_jiggler_min = value;
                }
              } else {
                if (value >= mj_jiggler_min) {
                  mj_jiggler_max = value;
                }
              }
            }
            break;
          case 'r': // Restore Default Timings
            mj_jiggler_min = MJ_DEFAULT_JIGGLER_MIN;
            mj_jiggler_max = MJ_DEFAULT_JIGGLER_MAX;
            [[fallthrough]];
          case 's':
            Serial.print(F("Jiggler delay min: "));
            Serial.print(mj_jiggler_min);
            Serial.print(F(" max: "));
            Serial.print(mj_jiggler_max);
            Serial.print(F("\r\n"));
            break;
          case 'S':
            mj_eeprom_write_int(MJ_EEPROM_JIGGLER_MIN, mj_jiggler_min);
            mj_eeprom_update_value(MJ_EEPROM_JIGGLER_MAX, 2, mj_jiggler_max);
            break;
          default:
            break;
        }
      }
      break;
    case 'p': // Jiggler Pixel maximum moves
      if (num >= 2) {
        switch (c[1]) {
          case '0':
          case '1':
          case '2':
          case '3':
          case '4':
          case '5':
          case '6':
          case '7':
          case '8':
          case '9':
            value = (byte) mj_convert_dec_long(c + 1);
            mj_jiggler_pixel = value;
            break;
          case 'S':
            mj_eeprom_update_value(MJ_EEPROM_JIGGLER_PIXEL, 1, mj_jiggler_pixel);
            break;
          case 'r':
            mj_jiggler_pixel = MJ_DEFAULT_JIGGLER_PIXEL;
            [[fallthrough]];
          case 's':
            Serial.print(F("Jiggler Pixel max: "));
            Serial.print(mj_jiggler_pixel);
            Serial.print(F("\r\n"));
            break;
          default:
            break;
        }
      }
      break;
  }
}

// Writes Baud-rate of serial line as on a atmega32u4 the Serial.begin(9600) does not really set baud.
void parsebaud(char *c, int num) {
  const char p[] = "NOEMS";
  const char s[][4] = { "1", "1.5", "2" };
  Serial.print(F("Baudrate is "));
  Serial.print(Serial.baud());
  Serial.print(F(" at "));
  Serial.print(Serial.numbits());
  Serial.print(F("/"));
  Serial.write(p[Serial.paritytype()]);
  Serial.print(F("/"));
  Serial.print(s[Serial.stopbits()]);  
  Serial.print(F(" RTS: "));
  Serial.print(Serial.rts());
  Serial.print(F("\r\n"));
}

void print_at_help() {
  parseinfo();
  Serial.print(F("Possible Commands - type at<command>? for more help\r\n"));
  Serial.print(F("ate - EEPROM\r\n"));
  Serial.print(F("atl - LED\r\n"));
  Serial.print(F("ati - Version Info\r\n"));
  Serial.print(F("atj - MouseJiggler\r\n"));
  Serial.print(F("att - AutoType\r\n"));
  Serial.print(F("atb - Input Buttons\r\n"));
  Serial.print(F("atr - Baud Rate\r\n"));
}

void print_atj_help() {
  parseinfo();
  Serial.print(F("atje - enable MouseJiggler\r\n"));
  Serial.print(F("atjd - disable MouseJiggler\r\n"));
  Serial.print(F("atjs - show Jiggler state\r\n"));
  Serial.print(F("atjS - Save Jiggler state to EEPROM\r\n"));
  Serial.print(F("atjr - restore default state\r\n"));
  Serial.print(F("atjp<number> - set maximal pixel move 0-255\r\n"));
  Serial.print(F("atjpr - restore default maximal pixel move\r\n"));
  Serial.print(F("atjps - show maximal pixel move\r\n"));
  Serial.print(F("atjpS - Save maximal pixel move to EEPROM\r\n"));
  Serial.print(F("atjt1<number> - set minimal time in ms between Jiggles\r\n"));
  Serial.print(F("atjt2<number> - set maximal time in ms between Jiggles\r\n"));
  Serial.print(F("atjtr - restore default minimal and maximal time\r\n"));
  Serial.print(F("atjts - show minimal and maximal time\r\n"));
  Serial.print(F("atjtS - Save minimal and maximal time to EEPROM\r\n"));
}

void print_atl_help() {
  parseinfo();
  Serial.print(F("atlp1<number> - set LED1 pin to <number>\r\n"));
  Serial.print(F("atlp2<number> - set LED2 pin to <number>\r\n"));
  Serial.print(F("atlps - show LED Pin assignments\r\n"));
  Serial.print(F("atlpS - Save LED Pin assignments to EEPROM\r\n"));
  Serial.print(F("atlpr - restore default LED Lin assignments\r\n"));
  Serial.print(F("       you cannot assign a pin that is already used by Buttons ot another LED\r\n"));
  Serial.print(F("atlt<1-4><0-65535) - set timing modifier <1-4> to <0-65535>"));
  Serial.print(F("                     10000 is default, higher is slower\r\n"));
  Serial.print(F("                     not all timings are used at every LED Mode\r\n"));
  Serial.print(F("atlts - show LED Timing modifier\r\n"));
  Serial.print(F("atlpS - Save LED Timing mofifier to EEPROM\r\n"));
  Serial.print(F("atlpS - restore default LED Timing mofifier\r\n"));
  Serial.print(F("atlm < 0 - 65535 > - set LED Blink Mode (only 0 - 7 are defined yet)\r\n"));
  Serial.print(F("atlms - show LED Blink Mode\r\n"));
  Serial.print(F("atlmS - Save LED Blink Mode to EEPROM\r\n"));
  Serial.print(F("atlmr - restore default LED Blink Mode\r\n"));
  Serial.print(F("atlo < 0 - ffff > - set LED Option. Only lowest 2 Bits are used atm. They invert LED1 or LED2 if set\r\n"));
  Serial.print(F("atlos - show LED Option\r\n"));
  Serial.print(F("atloS - Save LED Option to EEPROM\r\n"));
  Serial.print(F("atlor - restore default LED Option\r\n"));
}

void print_ati_help() {
  print_at_help();
}

void print_ate_help() {
  parseinfo();
  Serial.print(F("atec - Calculate CRC and show calculated and stored CRC\r\n"));
  Serial.print(F("ateC - Calculate CRC and update stored CRC\r\n"));
  Serial.print(F("ated - dump all EEPROM Contents\r\n"));
  Serial.print(F("atei - initialize defaults to EEPROM. Needs to be given three times to activate\r\n"));
  Serial.print(F("ateI - wipe and initialize defaults to EEPROM. Needs to be given three times to activate\r\n"));
  Serial.print(F("atez - wipe EEPROM. Only if you'd like to upload different Sketch. Needs to be given three times to activate\r\n"));
  Serial.print(F("atewb<xxx><yy> - write byte <yy> to EEPROM Address <xxx> - Data and Address in HEX\r\n"));
  Serial.print(F("atewi<xxx><yyyy> - write int <yyyy> to EEPROM Address <xxx> - Data and Address in HEX\r\n"));
  Serial.print(F("atewl<xxx><y-yyyyyyyy> - write long <y> to EEPROM Address <xxx> - Data and Address in HEX\r\n"));
  Serial.print(F("aterb<xxx> - read byte from Address <xxx>\r\n"));
  Serial.print(F("ateri<xxx> - read int from Address <xxx>\r\n"));
  Serial.print(F("aterl<xxx> - read long from Address <xxx>\r\n"));
}

void print_att_help() {
  parseinfo();
  Serial.print(F("atts - show string in EEPROM\r\n"));
  Serial.print(F("attk<string> - type given string to keyboard\r\n"));
  Serial.print(F("attw<string> - set string in EEPROM\r\n"));
  Serial.print(F("       following Escape-Codes exist for the string:\r\n"));
  Serial.print(F("       \\e -> ESC, \\n or \\r -> RETURN, \\t -> TAB, \\\\ -> \\,\r\n"));
  Serial.print(F("       \\d<0-9> Set typing delay to 0-9, \\x<HH> Hex code <HH>,\r\n"));
  Serial.print(F("       \\X<HH> same as \\x but without auto-remapping\r\n"));
  Serial.print(F("       \\s<HH> \"Special Key\" no press/release - hold the key \\sff releases all keys\r\n"));
  Serial.print(F("attt - type string in EEPROM to keyboard\r\n"));
  Serial.print(F("attd<0-9><number> - sets delay <0-9> to <number> (0-4294967295) in ms\r\n"));
  Serial.print(F("attds - show typing delays\r\n"));
  Serial.print(F("attdS - Save typing delays to EEPROM\r\n"));
  Serial.print(F("attdr - restore default typing delays\r\n"));
  Serial.print(F("attat<number> - set AutoType Delay to <number> (0-4294967295) ms\r\n"));
  Serial.print(F("        Will automatically type the string defined with attw after given Delay to Keyboard\r\n"));
  Serial.print(F("        The Delay counts from Power-On of the Stick not of the PC! Be careful about this!\r\n"));
  Serial.print(F("attae - enable AutoType - will only be active on reset so don't forget attaS\r\n"));
  Serial.print(F("attad - disable AutoType\r\n"));
  Serial.print(F("attas - show AutoType Delay\r\n"));
  Serial.print(F("attaS - Save AutoType Delay to EEPROM\r\n"));
  Serial.print(F("attar - restore default AutoType Delay\r\n"));
}

void print_atb_help() {
  parseinfo();
  Serial.print(F("atb<1-4><number> - sets button <1-4> to pin <number>\r\n"));
  Serial.print(F("atbs - shows button to pin assignments\r\n"));
  Serial.print(F("atbS - Save button to pin assignments to EEPROM\r\n"));
  Serial.print(F("atbr - restore default button to pin assignments\r\n"));
  Serial.print(F("       you cannot assign a pin that is already used by LED or another button\r\n"));
}

void print_atr_help() {
  parseinfo();
  Serial.print(F("atr - shows the baud-Rate the PC is connected with\r\n"));
}

// Parse string given via serial and pass thru different decoders
void parsestring(char *c, int num) {
  if (num > 2) {
    if (c[0] == 'a' && c[1] == 't') {  // String needs to start with 'at'
      if (num == 4) {
        if (c[3] == '?') {
          switch (c[2]) {
            case 'j':
              print_atj_help();
              break;
            case 'l':
              print_atl_help();
              break;
            case 'i':
              print_ati_help();
              break;
            case 'e':
              print_ate_help();
              break;
            case 't':
              print_att_help();
              break;
            case 'b':  // input buttons
              print_atb_help();
              break;
            case 'r':  // print actual baud rate
              print_atr_help();
              break;
            default:
              print_at_help();
              break;
          }
        }
      }
      switch (c[2]) {  // The following character decides which submodule will decode further
        case 'j':  // JIGGLER related
          parsejiggler(c + 3, num - 3);
          break;
        case 'l':  // LED related
          parseled(c + 3, num - 3);
          break;
        case 'i':  // Info Message
          parseinfo();
          break;
        case 'e':  // EEPROM related
          parseeeprom(c + 3, num - 3);
          break;
        case 't':  // autotype related
          parsekeyboard(c + 3, num - 3);
          break;
        case 'b':  // input buttons
          parsebuttons(c + 3, num - 3);
          break;
        case 'r':  // print actual baud rate
          parsebaud(c + 3, num - 3);
          break;
        default:
          print_at_help();
          break;
      }
    }
  } else {
    print_at_help();
  }
}
